/***********************************************************
* Projectile. Class to model a projectile travelling
* under the physics of the classic equations of motion.
* In particular, no air resistance.
*<p> Projectiles have an initial location in 2D from which
* they are launched. They also have an initial muzzle
* velocity and angle of launch. Methods are provided to
* track the flight of the projectile at an arbitrary
* time.
* @author Spike Barlow
* @date 26/7/1999
***********************************************************/
public class Projectile {

  private double initialHeight;
  private double initialHorizLocation;
  private double angleOfFire;
  private double muzzleVelocity;

  private static final double GRAVITY = 9.81;



/****************************************************************
* Construct a projectile with initial location and firing info.
* @param horiz the horizontal displacement of the launch site, in
* metres.
* @param height the height from which the projectile is launched
* in metres.
* @param angle the angle of launch in degrees.
* @param muzzle the muzzle velocity in metres/sec.
******************************************************************/
Projectile(double horiz, double height, double angle, double muzzle) {

  initialHeight = height;
  initialHorizLocation = horiz;
  angleOfFire = angle*Math.PI/180.0;
  muzzleVelocity = muzzle;
}



/****************************************************************
* Construct a projectile with initial location and firing info.
* The projectile is assumed to be launched from a location of (0,0).
* @param angle the angle of launch in degrees.
* @param muzzle the muzzle velocity in metres/sec.
******************************************************************/
Projectile(double angle, double muzzle) {

  this(0,0,angle,muzzle);
}




/******************************************************************
* Get the vertical position of the projectile at the specified
* time. In metres.
********************************************************************/
public double getVerticalPosition(double time) {

  if (time<=0)
    return initialHeight;
  else
    return initialHeight+muzzleVelocity*Math.sin(angleOfFire)*time
        - GRAVITY*time*time/2.0;
}



/********************************************************************
* Get the horizontal position of the projectile at the specified
* time. In metres.
*********************************************************************/
public double getHorizontalPosition(double time) {

  if (time<=0)
    return initialHorizLocation;
  else
    return initialHorizLocation + muzzleVelocity*Math.cos(angleOfFire)
                *time;
}



/*********************************************************************
* Get the instantaneous speed of the projectile at the specified
* time. In metres/second
**********************************************************************/
public double getInstantaneousSpeed(double time) {

  double vertSpeed = getVerticalSpeed(time);
  double horizSpeed = getHorizontalSpeed(time);
  return Math.sqrt(vertSpeed*vertSpeed + horizSpeed*horizSpeed);
}



/**********************************************************************
* Get the instantaneous direction (angle of flight) of the projectile
* at the specified time. In degrees.
***********************************************************************/
public double getInstantaneousDirection(double time) {

  double vertSpeed = getVerticalSpeed(time);
  double horizSpeed = getHorizontalSpeed(time);
  return Math.atan(vertSpeed/horizSpeed)*180.0/Math.PI;
}



/**********************************************************************
* Get the intial height from which the projectile is launched. In
* metres.
***********************************************************************/
public double getInitialHeight() {

  return initialHeight;
}



/***********************************************************************
* Get the initial horizontal position from which the projhectile is
* launched. In metres.
************************************************************************/
public double getInitialHorizLocation() {

  return initialHorizLocation;
}



/**********************************************************************
* Get the angle (in degrees) that the projectile was fired with.
***********************************************************************/
public double getAngleOfFire() {

  return angleOfFire*180.0/Math.PI;
}



/***********************************************************************
* Get the initial muzzle velocity with which the shell was fired. In
* metres/second.
************************************************************************/
public double getMuzzleVelocity() {

  return muzzleVelocity;
}




/************************************************************************
* get the vertical speed that the projectile is travelling at, at the
* specified time. In metres/second. Formula is the derivative of the
* distance formula with respect to time.
*************************************************************************/
private double getVerticalSpeed(double time) {

  if (time<=0)
    return 0;
  else
    return muzzleVelocity*Math.sin(angleOfFire) - GRAVITY*time;
}



/***********************************************************************
* Get the horizontal speed that the projectile is travelling at, at
* the specified time. In metres/sec.
************************************************************************/
private double getHorizontalSpeed(double time) {

  if (time<=0)
    return 0;
  else
    return muzzleVelocity*Math.cos(angleOfFire);
}
}


